macroScript constrainAB
	category:"Comet Cartoons"
	toolTip:"ConstrainAB: Constrains the last selected object to the first two, and creates a ABmix attribute on it."
	Icon:#("CometCartoons",18)
(
--
-- ConstrainAB.mcr - Michael B. Comet - comet@comet-cartoons.com
--					 http://www.comet-cartoons.com
--
-- This script will Point and Orient constraint the 3rd selected object
--		to the first two.  It adds a custom attribute onto the 3rd "slave" 
--		object that allows the constraints to be easily animated so the 
--		slave is between the 1st and 2nd "masterA" and "masterB" objects.
--
-- Version 1.00 - 02/18/02
--


/*
 * Global Variables:
 */
global cABStart = -1000;
global cABEnd = 10000;
mA = undefined;
mB = undefined;
sL = undefined;
cABDoPos = true;
cABDoRot = true;
cABDoPar = true;
cABDoACont = true;
cABDoBCont = true;
cABDoRotCons = false;

-- Functions -------------------------------------------------------------------------------------

/*
 * setTimeline() - This will set the timeline to the given start and end times
 */
fn setTimeline start end =
(
	animationRange = (interval start end);
)

-- -----------------------------------------------------------------------------------------------

/*
 * addConstrainABAttr() - Adds a constrainAB attribute to the given obj
 */
fn addConstrainABAttrs obj =
(
    if (obj == undefined) then 
		(
		format ("-- ConstrainAB INTERNAL ERROR: addConstrainABAttrs (), obj is undefined!\n");
		return();
		)

		-- Make sure user installed the AttributeHolder modifier.
    if (EmptyModifier != undefined) then 
		(
			-- Add Attr Holder modifier
		addModifier obj (EmptyModifier());
		)

		--
		-- main attr def stuff
		--
    str = "ConstrainABAttrs = attributes Custom_Attributes\n"+
		"(\n"+
	    "parameters main rollout:ConstrainABRO\n"+
		"	(\n"+
		"	constrainAB type:#float default:0.0 ui:constrainABSpin\n"+
		"	)\n"+
		"   \n"+
		"rollout ConstrainABRO \"ConstrainAB Parameters\"\n"+ 
		"   (\n"+
		"   spinner 'constrainABSpin' \"constrainAB\" type:#float range:[0,100,50];\n"+
	    "   )\n"+
		");\n"+
		"\n" ;
	execute str;

		--
		-- Add some custom def data like MAX does, so we can retrieve RANGE info later for things like my attrKey script.
		--
	str = "\n"+
		"pb1 = CAT_ParamBlock name:\"'constrainAB'\" type:\"Float\" ui:\"constrainABSpin\" default:\"50.0\";\n"+
		"ui1 = CAT_UIItem ui:\"Spinner\" name:\"'constrainAB'\" type:\"Float\" range:\"[0,100,0]\";\n"+ 
		"arr1 = #(pb1);\n"+
		"arr2 = #(ui1);\n"+
		"theArr = #(arr1, arr2)\n"+
		"\n"+
		"custAttributes.setDefData ConstrainABAttrs theArr;\n";
	execute str;
		
    if (EmptyModifier != undefined) then 
		str = "custAttributes.add $"+(obj.name)+".Attribute_Holder ConstrainABAttrs ;";
	else
		str = "custAttributes.add $"+(obj.name)+" ConstrainABAttrs ;";
	execute str;

		-- 
		-- Now add a default controller too and set value
		--	
    if (EmptyModifier != undefined) then 
		(
		str = "";
		str += "$"+(obj.name)+".Attribute_Holder.constrainAB.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".Attribute_Holder.constrainAB = 50;\n";
		)
	else
		(
		str = "";
		str += "$"+(obj.name)+".constrainAB.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".constrainAB = 50;\n";
		)
	execute str;
	
)



-- -----------------------------------------------------------------------------------------------

/*
 * constrainAB() - Given two master nodes, constrains the slave
 *		node with a pos and rot constraint between the two and 
 *		sets up a Custom Attribute on the slave to control 
 *		the mixing between A and B.
 */
fn constrainAB masterA masterB slave =
(
    if (masterA == undefined or masterB == undefined or slave == undefined) then
		return();

    format "-- constrainAB() - Constraining % to % & %.\n" slave.name masterA.name masterB.name;

    addConstrainABAttrs slave;		-- add the attrs

    if (cABDoPos) then
	    (
	    pCntl = slave.pos.controller = position_constraint();
		pCI = pCntl.constraints;		-- Get interface to constraints
		pCI.appendTarget masterA 50;
		pCI.appendTarget masterB 50;
	    )


    if (cABDoRotCons) then
	    (
	    rCntl = slave.rotation.controller = orientation_constraint();
		rCI = rCntl.constraints;		-- Get interface to constraints
		rCI.appendTarget masterA 50;
		rCI.appendTarget masterB 50;
		)

    -- Now we make the expression
    if (EmptyModifier != undefined) then 
		cA = ".Attribute_Holder";
	else
		cA = "";

    if (cABDoRot) then
	    (
		print "whee\n";


/*
--		tCntl = slave.transform.controller = transform_script();
	    rCntl = slave.rotation.controller = rotation_script();
		str = "";
		
		str += ("dependsOn $'"+masterA.name+"' $'"+masterB.name+"' $'"+slave.name+"'"+cA+".Custom_Attributes.constrainAB.controller\n");
	    str += ("coordsys world\n");
		str += ("    (\n");
		str += ("    mA = $'"+masterA.name+"'.transform.rotationPart as matrix3;\n");
		str += ("    mB = $'"+masterB.name+"'.transform.rotationPart as matrix3;\n");
		str += ("    pct = $'"+slave.name+"'"+cA+".Custom_Attributes.constrainAB;\n");
		str += ("    pct /= 100.0;  -- normalize to 0-1\n");
		str += ("	 pSA = point3 (1-pct) (1-pct) (1-pct); -- make scaling vectors\n");
		str += ("	 pSB = point3 pct pct pct; -- make scaling vectors\n");
		str += ("	 -- figure L and R parts of the equation\n");
		str += ("	 mL = scale mA pSA true;\n");
		str += ("    mR = scale mB pSB true;\n");
		str += ("    mT = mL + mR;  -- result is addition of each A and B parts\n");
		str += ("	 mT.rotationPart;\n");
		str += ("    )\n");
		str += ("\n");
		
--	    tCntl.script = str;
	    rCntl.script = str;
		)
*/

		-- For orientation/rotation we'll use a Script Controller with a MAXscript expression,
		-- since the orientation constraints will wig out and flip and give other problems.

	    rCntl = slave.rotation.controller = euler_xyz();

		rXCntl = slave.rotation.X_Rotation.controller = float_script();
		rYCntl = slave.rotation.Y_Rotation.controller = float_script();
		rZCntl = slave.rotation.Z_Rotation.controller = float_script();
	
		for i in 1 to 3 do
			(
		    str= "";
			str += ("dependsOn $'"+masterA.name+"' $'"+masterB.name+"' $'"+slave.name+"'"+cA+".Custom_Attributes.constrainAB.controller\n");
			str += ("\n");
			str += ("fn getPar obj =\n");
			str += ("(\n");
		    str += ("    if (obj== undefined) then\n");
		    str += ("        return (eulerangles 0 0 0);\n");
		    str += ("    v1 = getPar obj.parent;   -- recurse\n");
		    str += ("    coordsys world\n");
		    str += ("        (\n");
			str += ("        v2 = obj.rotation as eulerangles;\n");
		    str += ("        v2.x = (obj.rotation.x_rotation.controller.value)\n");
		 	str += ("        v2.y = (obj.rotation.y_rotation.controller.value)\n");
			str += ("        v2.z = (obj.rotation.z_rotation.controller.value)\n");
		    str += ("        )\n");
		    str += ("    v3 = eulerangles 0 0 0;\n");
		    str += ("    v3.x = v1.x + v2.x;\n");
		    str += ("    v3.y = v1.y + v2.y;\n");
		    str += ("    v3.z = v1.z + v2.z;\n");
		    str += ("    return v3;\n");
			str += (")\n");
			str += ("\n");
			str += ("rotA = getPar $'"+masterA.name+"';\n");
			str += ("rotB = getPar $'"+masterB.name+"';\n");
			str += ("\n");
			str += ("pct = $'"+slave.name+"'"+cA+".Custom_Attributes.constrainAB;\n");
			str += ("pct /= 100.0;  -- normalize to 0-1\n");
			str += ("rot = eulerangles 0 0 0 ;\n");
			str += ("rot.z = (( (1-pct)*rotA.z ) + ( (pct) * rotB.z ) );\n");
			str += ("rot.y = (( (1-pct)*rotA.y ) + ( (pct) * rotB.y ) );\n");
			str += ("rot.x = (( (1-pct)*rotA.x ) + ( (pct) * rotB.x ) );\n");

			if (i == 1) then 
				str += ("degToRad rot.x; \n");
			if (i == 2) then 
				str += ("degToRad rot.y; \n");
			if (i == 3) then 
				str += ("degToRad rot.z; \n");
			str += ("\n");
	
			if (i == 1) then 
			    rXCntl.script = str;
			if (i == 2) then 
			    rYCntl.script = str;
			if (i == 3) then 
			    rZCntl.script = str;
			)
	
/*
	
	    rCntl = slave.rotation.controller = rotation_script();
		
	    str= "";

		str += ("dependsOn $'"+masterA.name+"' $'"+masterB.name+"' $'"+slave.name+"'"+cA+".Custom_Attributes.constrainAB.controller\n");
	    str += ("coordsys world\n");
	    str += ("    (\n");
		str += ("    rotA = $'"+masterA.name+"'.rotation as eulerangles;\n");
		str += ("    rotA.x *= -1;\n");
		str += ("    rotA.y *= -1;\n");
		str += ("    rotA.z *= -1;\n");
		str += ("    rotB = $'"+masterB.name+"'.rotation as eulerangles;\n");
		str += ("    rotB.x *= -1;\n");
		str += ("    rotB.y *= -1;\n");
		str += ("    rotB.z *= -1;\n");
		str += ("    rotC = eulerangles 0 0 0;\n");
		str += ("    rotD = eulerangles 0 0 0;\n");
	    if (cABDoPar) then
		    (
		    str += ("    parentA = $'"+masterA.name+"'.parent;\n");
		    str += ("    parentB = $'"+masterB.name+"'.parent;\n");
		    str += ("	 while (parentA != undefined) do\n");
			str += ("	     (\n");
		    if (cABDoACont != true) then
				(
				str += ("        rotC = parentA.rotation as eulerangles;\n");
				str += ("         rotC.x *= -1;\n");
				str += ("         rotC.y *= -1;\n");
				str += ("         rotC.z *= -1;\n");
				)
			else (
				str += ("        rotC.z += parentA.rotation.z_rotation.controller.value;\n");
				str += ("        rotC.y += parentA.rotation.y_rotation.controller.value;\n");
				str += ("        rotC.x += parentA.rotation.x_rotation.controller.value;\n");
				)
			str += ("	     parentA = parentA.parent;  -- Keep going up hierarchy\n");
			str += ("	     )\n");
		    str += ("    while (parentB != undefined) do\n");
			str += ("	     (\n");
		    if (cABDoBCont != true) then
			    (
				str += ("        rotD = parentB.rotation as eulerangles;\n");
				str += ("        rotD.x *= -1;\n");
				str += ("         rotD.y *= -1;\n");
				str += ("         rotD.z *= -1;\n");
				)
			else (
				str += ("        rotD.z += parentB.rotation.z_rotation.controller.value;\n");
				str += ("        rotD.y += parentb.rotation.y_rotation.controller.value;\n");
				str += ("        rotD.x += parentB.rotation.x_rotation.controller.value;\n");
				)
			str += ("	     parentB = parentB.parent;  -- Keep going up hierarchy\n");
			str += ("	     )\n");
			)
		str += ("    -- Now get real values from the -controller- which reports true #'s greater than 180...\n");
	    if (cABDoACont) then
		    (
			str += ("    rotA.z = $'"+masterA.name+"'.rotation.z_rotation.controller.value;\n");
			str += ("    rotA.y = $'"+masterA.name+"'.rotation.y_rotation.controller.value;\n");
			str += ("    rotA.x = $'"+masterA.name+"'.rotation.x_rotation.controller.value;\n");
			)
	    if (cABDoBCont) then
		    (
			str += ("    rotB.z = $'"+masterB.name+"'.rotation.z_rotation.controller.value;\n");
			str += ("    rotB.y = $'"+masterB.name+"'.rotation.y_rotation.controller.value;\n");
			str += ("    rotB.x = $'"+masterB.name+"'.rotation.x_rotation.controller.value;\n");
			)
		str += ("    pct = $'"+slave.name+"'"+cA+".Custom_Attributes.constrainAB;\n");
		str += ("    pct /= 100.0;  -- normalize to 0-1\n");
		str += ("    rot = eulerangles 0 0 0 ;\n");
		str += ("    rot.z = (( (1-pct)*(rotA.z+rotC.z) ) + ( (pct) * (rotB.z+rotD.z) ) );\n");
		str += ("    rot.y = (( (1-pct)*(rotA.y+rotC.y) ) + ( (pct) * (rotB.y+rotD.y) ) );\n");
		str += ("    rot.x = (( (1-pct)*(rotA.x+rotC.x) ) + ( (pct) * (rotB.x+rotD.x) ) );\n");
		str += ("    rot; \n");
		str += ("    )\n");
		str += ("\n");

	    rCntl.script = str;

*/	
	
		)  -- end of cABDoRot	

    -- Now we wire it all up.
    if (EmptyModifier != undefined) then 
		cAB = slave.Attribute_Holder.Custom_Attributes;
	else
		cAB = slave.Custom_Attributes;


    if (cABDoPos) then
	    (
    	paramWire.connect cAB[#constrainAB] slave.position.controller[1] "100-constrainAB;";
	    paramWire.connect cAB[#constrainAB] slave.position.controller[2] "constrainAB;";
		)
    if (cABDoRotCons) then
	    (
	    paramWire.connect cAB[#constrainAB] slave.rotation.controller[1] "100-constrainAB;";
	    paramWire.connect cAB[#constrainAB] slave.rotation.controller[2] "constrainAB;";
		)

	clearSelection();
	select slave;
	
    format "-- constrainAB() Done.  Added Custom Attribute and Wired connections.\n";
)

-- -----------------------------------------------------------------------------------------------

-- Main Entry starts here!
/*
objs = getCurrentSelection();
cnt = objs.count;
if (cnt != 3) then 
    (
	format "-- ERROR: You must select 3 objects, with the last object being the slave.\n";
	)
else 
    (
	max create mode;	-- this speeds it up for some reason!
	setTimeline cABStart cABEnd;	-- Do this so our constraints and wiring don't die on frame 100...
    constrainAB objs[1] objs[2] objs[3];
	setTimeline 0 100;
	max modify mode;	-- show the new attr
    )


*/


fn filter_mA obj = 
(
	if (obj != mB and obj != sL) then
		return true;
	return false;
)

fn filter_mB obj = 
(
	if (obj != mA and obj != sL) then
		return true;
	return false;
)

fn filter_sL obj = 
(
	if (obj != mA and obj != mB) then
		return true;
	return false;
)


rollout cABMainRoll "Parameters"
(
    label la "masterA:" offset:[-30,3] across:2;
	pickbutton btn_mA "PICK MasterA" offset:[-20,0] tooltip:"Choose MasterA Object" width:150 align:#center filter:filter_mA
    label lb "masterB:" offset:[-30,3] across:2;
	pickbutton btn_mB "PICK MasterB" offset:[-20,0] tooltip:"Choose MasterB Object" width:150 align:#center filter:filter_mB
    label ls "slave:" offset:[-25,3] across:2;
	pickbutton btn_sL "PICK Slave" offset:[-20,0] tooltip:"Choose Slave Object" width:150 align:#center filter:filter_sL

	button btn_docAB "ConstrainAB" tooltip:"Does the ConstrainAB" width:214 align:#center enabled:false

	group "Options"
	    (
		checkbox cb_DoPos "Position via Constraint" checked:true align:#left;
		checkbox cb_DoRotCons "Rotation via Constraint  (A or B has IK)" checked:false align:#left;
		checkbox cb_DoRot     "Rotation via Script  (A and B are normal)" checked:true align:#left;
		checkbox cb_DoPar "Include Parent Rot." checked:true align:#left;
		checkbox cb_DoACont "A reads True Controller Values" checked:true align:#left;
		checkbox cb_DoBCont "B reads True Controller Values" checked:true align:#left;
		)
	on btn_mA picked obj do 
	    (
		mA = obj;
		btn_mA.text = obj.name;
		if (mA != undefined and mB != undefined and sL != undefined) then (
			btn_docAB .enabled = true;
			)
	    )

	on btn_mB picked obj do 
	    (
		mB = obj;
		btn_mB.text = obj.name;
		if (mA != undefined and mB != undefined and sL != undefined) then (
			btn_docAB .enabled = true;
			)
	    )

	on btn_sL picked obj do 
	    (
		sL = obj;
		btn_sL.text = obj.name;
		if (mA != undefined and mB != undefined and sL != undefined) then (
			btn_docAB .enabled = true;
			)
	    )

	on cb_DoPos changed state do
		(
		cABDoPos = state;
		)


	on cb_DoPar changed state do
		(
		cABDoPar = state;
		)

	on cb_DoACont changed state do
		(
		cABDoACont = state;
		)

	on cb_DoBCont changed state do
		(
		cABDoBCont = state;
		)

	on cb_DoRot changed state do
		(
		cABDoRot = state;
        if (state) then 
		    (
			cABDoRotCons = false;
			cb_DoPar.enabled = true;
			cb_DoACont.enabled = true;
			cb_DoBCont.enabled = true;
			)
		else 
			(
			cABDoRotCons = true;
			cb_DoPar.enabled = false;
			cb_DoACont.enabled = false;
			cb_DoBCont.enabled = false;
			)
		cb_DoRotCons.state = cABDoRotCons;
		)

	on cb_DoRotCons changed state do
		(
		cABDoRotCons = state;
		if (state) then
			(
			cABDoRot = false;
			cb_DoPar.enabled = false;
			cb_DoACont.enabled = false;
			cb_DoBCont.enabled = false;
			)
		else 
		    (
			cABDoRot = true;
			cb_DoPar.enabled = true;
			cb_DoACont.enabled = true;
			cb_DoBCont.enabled = true;
			)
		cb_DoRot.state = cABDoRot;
		)


    on btn_docAB pressed do
		(
		max create mode;	-- this speeds it up for some reason!
		setTimeline cABStart cABEnd;	-- Do this so our constraints and wiring don't die on frame 100...
	    constrainAB mA mB sL;
		setTimeline 0 100;
		max modify mode;	-- show the new attr
		)
		
)

rollout cABAboutRoll "About"
(
	label abt_1 "constrainAB" align:#center
	label abt_2 "v. 2.00" align:#center
	label abt_3 "By" align:#center
	label abt_4 "Michael B. Comet" align:#center
	label abt_5 "comet@comet-cartoons.com" align:#center
	label abt_6 "http://www.comet-cartoons.com/" align:#center
)


rolloutcABFloater = newRolloutFloater "constrainAB" 260 340;
addRollout cABMainRoll rolloutcABFloater;
addRollout cABAboutRoll rolloutcABFloater rolledUp:true;


)